import * as entityForm from '../entity-form.js';
import * as fieldPluginBase from './field-plugin-base.js';

/**
 * Default field plugin.
 */
export class DefaultPlugin extends fieldPluginBase.FieldPluginBase {

  supportedWidgets = [];

  focusElement(EditableElement) {
    this.originalValue = this.getInputValue(EditableElement);
    this.getFormItemInputs(EditableElement)[0].focus();
  }

  blurElement(EditableElement) {
    const inputValue = this.getInputValue(EditableElement);
    this.replaceEditableElementWithMarkup(EditableElement);
    if (JSON.stringify(this.originalValue) !== JSON.stringify(inputValue)) {
      entityForm.updateTempstore(EditableElement, inputValue.length === 0);
    }
    this.originalValue = null;
  }

  getFormItemInputs(EditableElement) {
    return EditableElement.getFormItemInputs();
  }

  getInputValue(EditableElement) {
    const inputs = this.getFormItemInputs(EditableElement);
    if (inputs[0].type === 'checkbox') {
      let checkboxes = [];
      for (let i=0; i < inputs.length; i++) {
        const value = inputs[i].checked ? inputs[i].value : null;
        if (value) {
          checkboxes.push(value);
        }
      }
      return checkboxes;
    }
    else {
      let values = {};
      for (let i=0; i < inputs.length; i++) {
        if (inputs[i].value) {
          values[i] = inputs[i].value;
        }
      }
      return values;
    }
  }

}

export default function ($ = jQuery, Drupal, once, dropZones) {

  /**
   * Register the default field plugin.
   */
  window.addEventListener('EditPlusFieldManager.RegisterPlugins', e => {
    e.detail.manager.registerPlugin(new DefaultPlugin());
  });

}
