<?php declare(strict_types = 1);

namespace Drupal\edit_plus\Controller;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Messenger\MessengerTrait;
use Drupal\edit_plus\EditPlusMessagesTrait;
use Symfony\Component\HttpFoundation\Request;
use Drupal\edit_plus\EditPlusTempstoreRepository;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Edit + tempstore controller.
 */
class Tempstore extends ControllerBase {

  use MessengerTrait;
  use EditPlusMessagesTrait;

  public function __construct(
    protected EditPlusTempstoreRepository $tempstoreRepository
  ) {}

  public static function create(ContainerInterface $container) {
    return new static (
      $container->get('edit_plus.tempstore_repository'),
    );
  }

  /**
   * Save tempstore entities.
   */
  public function save(Request $request) {
    $this->loopThroughEntities($request, function ($entity) {
      $tempstore_entity = $this->tempstoreRepository->get($entity);
      $tempstore_entity->save();
      $this->tempstoreRepository->delete($entity);
    });

    return new RedirectResponse(\Drupal::destination()->get());
  }

  /**
   * Delete tempstore entities.
   */
  public function delete(Request $request) {
    $this->loopThroughEntities($request, function ($entity) {
      $this->tempstoreRepository->delete($entity);
    });

    $this->clearUnsavedChangesMessage();

    return new RedirectResponse(\Drupal::destination()->get());
  }

  /**
   * Loop through entities.
   *
   * Gets the relevant entities from the request and loops through them.
   *
   * @param $request
   *   The current request.
   * @param $callable
   *   The function to call. @see delete() or save().
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  private function loopThroughEntities($request, $callable) {
    $entities = explode('::', $request->get('entities'));
    if (empty($entities)) {
      throw new \InvalidArgumentException('No entities to save.');
    }
    foreach ($entities as $entity) {
      [$entity_type_id, $entity_id] = explode('.', $entity);
      $entity = $this->entityTypeManager()->getStorage($entity_type_id)->load($entity_id);
      Cache::invalidateTags([getCacheTag($entity)]);
      $callable($entity);
    }
  }

}
