<?php

declare(strict_types=1);

namespace Drupal\edit_plus\EventSubscriber;

use Drupal\edit_plus\Event\FieldAttributes;
use Drupal\Core\Field\FieldDefinitionInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Clone field attribute.
 */
class CloneFieldAttribute implements EventSubscriberInterface {

  const DO_NOT_CLONE = [
    'entity_autocomplete',
    'inline_textarea',
  ];

  public function addFormItemAttributesAlter(FieldAttributes $event) {
    $field_name = $event->getFieldName();
    $form = $event->getForm();
    $form_item = &$form[$field_name];

    if (empty($form_item['#attributes']['data-edit-plus-clone'])) {
      $field_definition = $event->getEntity()->getFieldDefinition($field_name);
      $clone = $this->getClonedSetting($field_definition) ?? $this->getCloneDefault($form_item, $field_definition);
      $form_item['#attributes']['data-edit-plus-clone'] = $clone;
      $event->setForm($form);
    }

  }

  private function getClonedSetting(FieldDefinitionInterface $field_definition) {
    if (method_exists($field_definition, 'getThirdPartySettings')) {
      $third_party_settings = $field_definition->getThirdPartySettings('edit_plus');
      if (!empty($third_party_settings['clone'])) {
        return $third_party_settings['clone'];
      }
    }
    return NULL;
  }

  private function getCloneDefault(array $form_item, FieldDefinitionInterface $field_definition) {
    $type = edit_plus_find_widget_type($form_item, $field_definition);
    if (!empty($type) && in_array($type, self::DO_NOT_CLONE)) {
      return 'false';
    }
    return 'true';
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      FieldAttributes::ALTER => ['addFormItemAttributesAlter'],
    ];
  }

}
