<?php declare(strict_types = 1);

namespace Drupal\edit_plus\EventSubscriber;

use Drupal\Core\Render\Markup;
use Drupal\twig_events\Event\TwigRenderTemplateEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

final class EntityTemplate implements EventSubscriberInterface {

  public function onTwigRenderTemplate(TwigRenderTemplateEvent $event): void {
    $variables = $event->getVariables();
    $entity_info = $this->getEntityInfo($variables);
    if (!empty($entity_info)) {
      // Give the rendered entity an AJAX wrapper so it can be updated as
      // changes are made.
      $output = $event->getOutput();
      $wrapped_output = sprintf('<div class="edit-plus-entity-wrapper" data-edit-plus-entity-wrapper="%s::%s::%s::%s">%s</div>', $entity_info['entity_type'], $entity_info['entity_id'], $entity_info['view_mode'], $entity_info['bundle'], $output->__toString());
      $event->setOutput(Markup::create($wrapped_output));
    }
  }


  private function getEntityInfo(array $variables) {
    if (!empty($variables['elements']['#edit_plus_entity'])) {
      // Regular entities.
      return $variables['elements']['#edit_plus_entity'];
    } elseif (!empty($variables['elements']['content']['#edit_plus_entity'])) {
      // Inline blocks.
      return $variables['elements']['content']['#edit_plus_entity'];
    }
  }


  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      TwigRenderTemplateEvent::class => ['onTwigRenderTemplate'],
    ];
  }

}
