<?php

declare(strict_types=1);

namespace Drupal\edit_plus\EventSubscriber;

use Drupal\edit_plus\Event\FieldAttributes;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Default field attributes.
 */
class MediaFieldAttributes implements EventSubscriberInterface {

  /**
   * Add form item attributes.
   *
   * Add attributes that associate the form element with the rendered
   * page markup.
   *
   * @param \Drupal\edit_plus\Event\FieldAttributes $event
   *   The event.
   */
  public function addFormItemAttributesAlter(FieldAttributes $event) {
    if (self::isMediaReferenceField($event)) {
      $entity = $event->getEntity();
      $field_name = $event->getFieldName();
      $form = $event->getForm();
      $form_item = &$form[$field_name];

      $form_item['#attributes']['class'][] = 'edit-plus-hidden';
      $form_item['#attributes']['class'][] = 'edit-plus-form-item-wrapper';
      $form_item['#attributes']['data-edit-plus-handle'] = 'wrapper';
      $form_item['#attributes']['data-edit-plus-clone'] = 'true';
      $form_item['#attributes']['data-edit-plus-form-item-wrapper-id'] = sprintf('%s::%s::%s::%s', $entity->getEntityTypeId(), edit_plus_entity_identifier($entity), $field_name, 'value');
      $form_item['widget']['#type'] = 'container';
      $form_item['widget']['#attributes']['class'][] = 'edit-plus-form-item';
      $form_item['widget']['#attributes']['data-edit-plus-form-item-id'] = sprintf('%s::%s::%s::%s::%s', $entity->getEntityTypeId(), edit_plus_entity_identifier($entity), $field_name, 0, 'value');
      $form_item['widget']['#attributes']['data-edit-plus-form-item-widget'] = edit_plus_find_widget_type($form_item, $entity->getFieldDefinition($field_name));

      $event->setForm($form);
      $event->stopPropagation();
    }
  }

  public static function addFormItemAttributesAfterBuild(FieldAttributes $event) {
    if (self::isMediaReferenceField($event)) {
      $event->stopPropagation();
    }
  }

  /**
   * Is media field.
   *
   * @param \Drupal\edit_plus\Event\FieldAttributes $event
   *   The FieldAttributes event.
   *
   * @return bool
   *   Whether the field is a Media field.
   */
  public static function isMediaReferenceField(FieldAttributes $event): bool {
    $entity = $event->getEntity();
    $field_name = $event->getFieldName();
    $field_definition = $entity->getFieldDefinition($field_name);
    $target_type = $field_definition->getFieldStorageDefinition()->getSetting('target_type');
    $form = $event->getForm();
    return $target_type === 'media' &&
      !empty($form[$field_name]['widget']['selection'][0]['#theme']) &&
      $form[$field_name]['widget']['selection'][0]['#theme'] === 'media_library_item__widget'
    ;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      FieldAttributes::ALTER => ['addFormItemAttributesAlter', 100],
      FieldAttributes::AFTER_BUILD => ['addFormItemAttributesAfterBuild', 100],
    ];
  }

}
