<?php

namespace Drupal\edit_plus\ParamConverter;

use Drupal\Core\Messenger\MessengerInterface;
use Drupal\edit_plus\EditPlusTempstoreRepository;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\ParamConverter\ParamConverterInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\ParamConverter\EntityConverter as EntityConverterBase;

/**
 * Entity Converter.
 *
 * Uses the Edit + tempstore version of an entity if it exist.
 */
class EntityConverter extends EntityConverterBase {

  use StringTranslationTrait;

  public function __construct(
    protected ParamConverterInterface $inner,
    EntityTypeManagerInterface $entity_type_manager,
    EntityRepositoryInterface $entity_repository,
    protected EditPlusTempstoreRepository $editPlusTempstoreRepository,
    protected MessengerInterface $messenger
  ) {
    parent::__construct($entity_type_manager, $entity_repository);
  }

  /**
   * {@inheritdoc}
   */
  public function convert($value, $definition, $name, array $defaults) {
    $entity = parent::convert($value, $definition, $name, $defaults);

    // Use the Edit + tempstore version of an entity if it exist.
    $tempstore_entity = $this->editPlusTempstoreRepository->get($entity);
    if ($tempstore_entity && $entity !== $tempstore_entity) {
      $entity = $tempstore_entity;
      // Flag that we have swapped this entity with a tempstore version.
      edit_plus_active_tempstore_entities($entity);
      $this->messenger->addWarning($this->t('You have unsaved changes.'));
    }

    return $entity;
  }

}
